/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.operator.common.elasticsearch;

import static org.assertj.core.api.Assertions.*;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import co.elastic.clients.elasticsearch.ElasticsearchClient;
import de.ozgcloud.operator.IndicesPrivilegesDataTestFactory;
import de.ozgcloud.operator.PutRoleRequestDataTestFactory;
import de.ozgcloud.operator.PutUserRequestDataTestFactory;

class ElasticsearchRemoteServiceTest {

	@Spy
	@InjectMocks
	private ElasticsearchRemoteService service;
	@Mock
	private ElasticsearchClient client;
	
	@DisplayName("Create security role")
	@Nested
	class TestCreateSecurityRole {
		
		@DisplayName("create put role request")
		@Nested
		class TestCreatePutRoleRequest {
			
			@Test
			void shouldHaveName() {
				var request = service.createPutRoleRequest(PutRoleRequestDataTestFactory.create());
				
				assertThat(request.name()).isEqualTo(PutRoleRequestDataTestFactory.NAME);
			}
			
			@DisplayName("indices privileges")
			@Nested
			class TestIndicesPrivileges {
				
				@Test
				void shouldHaveName() {
					var request = service.createPutRoleRequest(PutRoleRequestDataTestFactory.create());
					
					assertThat(request.indices().get(0).names()).containsExactly(IndicesPrivilegesDataTestFactory.NAME);
				}
				
				@Test
				void shouldHavePrivileges() {
					var request = service.createPutRoleRequest(PutRoleRequestDataTestFactory.create());
					
					assertThat(request.indices().get(0).privileges()).containsExactly(IndicesPrivilegesDataTestFactory.PRIVILEGES);
				}
			}
		}
	}
	
	@DisplayName("Create security user")
	@Nested
	class TestCreateSecurityUser {
		
		@DisplayName("create put user request")
		@Nested
		class TestCreatePutUserRequest {
			
			@Test
			void shouldHaveName() {
				var request = service.createPutUserRequest(PutUserRequestDataTestFactory.create());
				
				assertThat(request.username()).isEqualTo(PutUserRequestDataTestFactory.USERNAME);
			}
			
			@Test
			void shouldHaveRoles() {
				var request = service.createPutUserRequest(PutUserRequestDataTestFactory.create());
				
				assertThat(request.roles()).containsExactly(PutUserRequestDataTestFactory.ROLES);
			}
			
			@Test
			void shouldHavePassword() {
				var request = service.createPutUserRequest(PutUserRequestDataTestFactory.create());
				
				assertThat(request.password()).isEqualTo(PutUserRequestDataTestFactory.PASSWORD);
			}
		}
	}
}