/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.operator.common.elasticsearch;

import java.time.Duration;
import java.time.temporal.ChronoUnit;

import org.apache.http.HttpHost;
import org.apache.http.auth.AuthScope;
import org.apache.http.auth.UsernamePasswordCredentials;
import org.apache.http.impl.client.BasicCredentialsProvider;
import org.elasticsearch.client.RestClient;
import org.testcontainers.elasticsearch.ElasticsearchContainer;

import co.elastic.clients.elasticsearch.ElasticsearchClient;
import co.elastic.clients.json.jackson.JacksonJsonpMapper;
import co.elastic.clients.transport.rest_client.RestClientTransport;

public class ElasticsearchTestClient {
	
	private static final String IMAGE_NAME = "docker.elastic.co/elasticsearch/elasticsearch:8.16.1";
	private static final String USER = "elastic";
	private static final String PASSWORD = "s3cret";
	private static final int PORT = 9200;
	private static final String SCHEME = "https";
	private static final Duration STARTUP_TIMEOUT = Duration.of(2, ChronoUnit.MINUTES);
	 
	public static final ElasticsearchContainer ELASTICSEARCH_CONTAINER = new ElasticsearchContainer(IMAGE_NAME)
			.withExposedPorts(PORT)
			.withPassword(PASSWORD)
			.withStartupTimeout(STARTUP_TIMEOUT);

	public static ElasticsearchClient create() {
		var transport = new RestClientTransport(buildRestClient(), new JacksonJsonpMapper());
		return new ElasticsearchClient(transport);
	}
	
	private static RestClient buildRestClient() {
		var host = new HttpHost("localhost", ELASTICSEARCH_CONTAINER.getMappedPort(PORT), SCHEME);
		var credentialsProvider = buildCredentialsProvider();
		var builder = RestClient.builder(host);

		builder.setHttpClientConfigCallback(clientBuilder -> {
			clientBuilder.setSSLContext(ELASTICSEARCH_CONTAINER.createSslContextFromCa());
			clientBuilder.setDefaultCredentialsProvider(credentialsProvider);
			return clientBuilder;
		});
//		builder.setNodeSelector(INGEST_NODE_SELECTOR);
//        final ObjectMapper mapper = new ObjectMapper();
//        mapper.setPropertyNamingStrategy(PropertyNamingStrategies.SNAKE_CASE);
//        mapper.setSerializationInclusion(JsonInclude.Include.NON_NULL);
		return builder.build();
	}
	
	private static BasicCredentialsProvider buildCredentialsProvider() {
		var credentialsProvider = new BasicCredentialsProvider();
		credentialsProvider.setCredentials(AuthScope.ANY, new UsernamePasswordCredentials(USER, PASSWORD));
		return credentialsProvider;
	}

}
