/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.operator.common.kubernetes;

import static org.assertj.core.api.Assertions.*;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;

import io.fabric8.kubernetes.client.KubernetesClient;
import io.fabric8.kubernetes.client.extension.ResourceAdapter;
import io.javaoperatorsdk.jenvtest.junit.EnableKubeAPIServer;

@EnableKubeAPIServer
class KubernetesRemoteServiceITCase {

	private static KubernetesClient client;
	private final KubernetesRemoteService service = new KubernetesRemoteService(client);

	@DisplayName("Get secret resource")
	@Nested
	class TestGetSecretResource {
		
		@Test
		void shouldReturnEmptyResourceIfMissing() {
			var resource = service.getSecretResource(NamespaceTestFactory.NAMESPACE, SecretTestFactory.NAME);
			
			assertThat(resource.get()).isNull();
		}
		
		@Test
		void shouldReturnSecretResourceIfExists() {
			createNamespace();
			createSecret();
			
			var resource = service.getSecretResource(NamespaceTestFactory.NAMESPACE, SecretTestFactory.NAME);
			
			assertThat(resource.get()).isNotNull();
		}
		
		private void createNamespace() {
			var resource = client.namespaces().withName(NamespaceTestFactory.NAMESPACE);
			var adapter = new ResourceAdapter<>(resource);
			adapter.create(NamespaceTestFactory.create());
		}
		
		private void createSecret() {
			var resource = client.secrets().inNamespace(NamespaceTestFactory.NAMESPACE).withName(SecretTestFactory.NAME);
			var adapter = new ResourceAdapter<>(resource);
			adapter.create(SecretTestFactory.create());
		}
	}
}